
#-++++++++++++++++++++++++++++++++++++++++++
# Basic info about addon
#-------------------------------------------
bl_info = {
    "name": "Maniaplanet Addon",
    "author": "Mladen Gabic",
    "version": (1, 8),
    "blender": (2, 76, 0),
    "location": "View3D > Tools > Maniaplanet Addon",
    "description": "Prepares object for maniaplanet importer",
    "category": "3d View"
    }


#-++++++++++++++++++++++++++++++++++++++++++
# Import some namespaces
#-------------------------------------------
import bpy
import os
import shutil
from bpy.props import *


#-++++++++++++++++++++++++++++++++++++++++++
# Get path to the directory where blender file is stored
#-------------------------------------------
def getDirectoryPath(ctx):
    fbxPath = ctx.scene.my_prop_grp.exPath
    if ctx.scene.my_prop_grp.sProp:
        fbxPath += ctx.scene.name + "\\"
    
    if not os.path.exists(fbxPath):
        os.makedirs(fbxPath)
    
    return fbxPath

def isObjectFirstLod(objName):
    if (objName.lower()).endswith("lod",0, len(objName)-1):
        return True
    else:
        return False

#-++++++++++++++++++++++++++++++++++++++++++
# Function that Exports fbx
#-------------------------------------------
def exportFbx(ctx, self):
    
    lodTxt = ""
    for obj in bpy.context.selectable_objects:
        lodTxt = ""
        obj2 = obj
        # unselect all  
        for item in bpy.context.selectable_objects:  
            item.select = False
        if isObjectFirstLod(obj.name):
            if not obj.name.endswith("0"):
                continue
            else:
                lodTxt = "_Lod0"
                obj2 = bpy.data.objects[obj.name[:-1] + "1"]
                obj2.select = True
        
        bpy.context.scene.objects.active = obj
        obj.select = True 
        
        fixMats(ctx)
        
        fbxPath = getDirectoryPath(ctx)
        
        finalPath = ""
        scaleFactor = 2
        
        # If its lod then remove lod text from it
        if lodTxt == "":
            objOriginalName = obj.name
        else:
            objOriginalName = obj.name[:-5]
        namePrefix= objOriginalName
        extension = ".fbx"
        nameEndfix= ""
        
        # Send warning to use if he did not save blender file
        if fbxPath == "":
            self.report({'ERROR'}, "You need to specify export path first")
            return {'CANCELLED'}
        
        # Save current location of object
        objX = obj.location[0]
        objY = obj.location[1]
        objZ = obj.location[2]
        # put object in center of the world
        obj.location[0] = 0
        obj.location[1] = 0
        obj.location[2] = 0
        # position secound lod if this is lod
        if not lodTxt == "":
            obj2.location[0] = 0
            obj2.location[1] = 0
            obj2.location[2] = 0
        
        numOfScales = 0
        curentScale = 0
        if ctx.scene.my_prop_grp.bProp == True:
            if objOriginalName[len(objOriginalName)-2 : len(objOriginalName)-1] == "#":
                namePrefix= objOriginalName[0 : len(objOriginalName) - 2]
                try:
                    int(objOriginalName[len(objOriginalName)-1 : len(objOriginalName)])
                except ValueError:
                    self.report({'ERROR'}, "Scale should be number")
                    return {'CANCELLED'}
                curentScale = int(objOriginalName[len(objOriginalName)-1 : len(objOriginalName )])
                numOfScales = curentScale - ctx.scene.my_prop_grp.iProp
                if numOfScales < 0:
                    self.report({'ERROR'}, "Min scale is bigger then object scale")
                    return {'CANCELLED'}
            else:
                self.report({'ERROR'}, "Scale number not found")
                return {'CANCELLED'}

        for x in range(0, numOfScales + 1):
            if ctx.scene.my_prop_grp.bProp == True:
                nameEndfix = "#" + str(curentScale - x)
            finalPath=fbxPath + namePrefix + nameEndfix + extension
            scaleFactor = scaleFactor/2
            obj.scale[0] = scaleFactor
            obj.scale[1] = scaleFactor
            obj.scale[2] = scaleFactor
            # Change scale of second object if first is lod
            if not lodTxt == "":
                obj2.scale[0] = scaleFactor
                obj2.scale[1] = scaleFactor
                obj2.scale[2] = scaleFactor
                obj2.name = namePrefix + nameEndfix + "_Lod1"
            
            # Rename object before exporting to new name
            obj.name = namePrefix + nameEndfix + lodTxt
              
            # Possible versions: ASCII6100 or BIN7400
            # Expodt fbx function with arguments
            bpy.ops.export_scene.fbx(filepath=finalPath, check_existing=False, axis_forward='-Z', axis_up='Y', filter_glob="*.fbx", version='BIN7400', ui_tab='MAIN', use_selection=True, global_scale=1.0, apply_unit_scale=False, bake_space_transform=True, object_types={'EMPTY', 'LAMP', 'MESH'}, use_mesh_modifiers=True, mesh_smooth_type='FACE', use_mesh_edges=False, use_tspace=False, use_custom_props=False, add_leaf_bones=False, primary_bone_axis='Y', secondary_bone_axis='X', use_armature_deform_only=False, bake_anim=False, bake_anim_use_all_bones=False, bake_anim_use_nla_strips=False, bake_anim_use_all_actions=False, bake_anim_force_startend_keying=False, bake_anim_step=1.0, bake_anim_simplify_factor=1.0, use_anim=False, use_anim_action_all=False, use_default_take=True, use_anim_optimize=False, anim_optimize_precision=6.0, path_mode='AUTO', embed_textures=False, batch_mode='OFF', use_batch_own_dir=True, use_metadata=False)
            # Return to old name
            obj.name = objOriginalName + lodTxt
            
        # return object back to its old position
        obj.location[0] = objX
        obj.location[1] = objY
        obj.location[2] = objZ
        obj.scale[0] = 1
        obj.scale[1] = 1
        obj.scale[2] = 1
        
        if not lodTxt == "":
                obj2.name = objOriginalName + "_Lod1"
                obj2.location[0] = objX
                obj2.location[1] = objY
                obj2.location[2] = objZ
                obj2.scale[0] = 1
                obj2.scale[1] = 1
                obj2.scale[2] = 1
        
    return {'FINISHED'}


#-++++++++++++++++++++++++++++++++++++++++++
# Function that renders Icon
#-------------------------------------------
def renderIcon(ctx,self):
        obj = ctx.active_object
        viewtype = ctx.scene.ViewType
        renderIconSize = ctx.scene.RenderIconSize
        bpy.ops.object.mode_set(mode='OBJECT')
        # set up render settings for icon
        bpy.context.scene.render.image_settings.color_mode = 'RGBA'
        bpy.context.scene.render.image_settings.file_format = 'TARGA'
        bpy.context.scene.render.alpha_mode = 'TRANSPARENT'
        bpy.context.scene.render.resolution_percentage = 100
        imagePath = getDirectoryPath(ctx)
        # Send warning to use if he did not save blender file
        if imagePath == "":
            self.report({'ERROR'}, "You need to specify export path first")
            return {'CANCELLED'}
        imagePath += "Icon\\" + obj.name + ".tga"
        bpy.context.scene.render.filepath = imagePath
        # set dictionary from where we match enum to actual icon size
        icoSizesDict = {"id_0":16, "id_1":32, "id_2":64, "id_3":128, "id_4":256}
        bpy.context.scene.render.resolution_x = icoSizesDict[renderIconSize]
        bpy.context.scene.render.resolution_y = icoSizesDict[renderIconSize]
        
        # Remove all cameras from the scene
        for ob in bpy.context.scene.objects:
            if ob.type=="CAMERA":
                ob.select = True
            else:
                ob.select = False           
        bpy.ops.object.delete()
        
        #add new camera
        lastObjectName = obj.name
        cam = None
        
        # Tweak camera if viewtype is current or else if it is closest
        if viewtype== "id_0":
            cam = bpy.ops.object.camera_add()
            bpy.ops.view3d.camera_to_view()
            bpy.ops.render.render(write_still=True)
            bpy.ops.object.delete()
            bpy.ops.object.select_pattern(pattern=obj.name)
            bpy.context.scene.objects.active = obj
        else:
            # Do some mojo to put camera as close as posible to the object
            bpy.ops.object.origin_set(type = 'ORIGIN_GEOMETRY')
            obj.location[0] = 0
            obj.location[1] = 0
            obj.location[2] = 0
            
            cam = bpy.ops.object.camera_add(location = obj.dimensions + obj.location)
            bpy.ops.object.constraint_add(type='TRACK_TO')
            bpy.context.object.constraints["Track To"].target = bpy.data.objects[lastObjectName]
            bpy.context.object.constraints["Track To"].up_axis = 'UP_Y'
            bpy.context.object.constraints["Track To"].track_axis = 'TRACK_NEGATIVE_Z'
            bpy.context.object.data.lens = 28
            bpy.ops.render.render(write_still=True)
        return {'FINISHED'}


#-++++++++++++++++++++++++++++++++++++++++++
# Fix material names
#-------------------------------------------
def fixMats(ctx):
    obj = ctx.active_object
    mats = obj.material_slots
    for mat in mats:
        fixedName = ""
        nameParts = mat.name.split('.')
        # if material dont need fixing continue
        if len(nameParts) > 1:
            for i in range(0, len(nameParts) - 1):
                fixedName += nameParts[i]
                if i + 1 < len(nameParts) - 1:
                    fixedName += "."
            # Get editable material
            if bpy.data.materials.get(mat.name) is not None:
                matPublic = bpy.data.materials[mat.name]
                matPublic.name = "someFakeCrap"
                matPublic.name = fixedName
                print("NEW MAT NAME IS: " + fixedName)
                print("REAL MAT NAME IS: " + matPublic.name)  
    return {'FINISHED'}

#-++++++++++++++++++++++++++++++++++++++++++
# Fix vertecies that have too many decimals
#-------------------------------------------
def fixVerts(ctx):
    mesh = ctx.active_object.data
    for vert in mesh.vertices:
        #vert.co.x = 0
        #vert.co.y = 0
        #vert.co.z = 0
        
        vert.co.x = round(vert.co.x, 3)
        vert.co.y = round(vert.co.y, 3)
        vert.co.z = round(vert.co.z, 3)  
        print( 'v %f %f %f\n' % (vert.co.x, vert.co.y, vert.co.z) )
    
    
    return {'FINISHED'}

#-++++++++++++++++++++++++++++++++++++++++++
# Create material and give it correct name
#-------------------------------------------
def createMat(ctx):
    obj = ctx.active_object
    mName = ctx.scene.Mat_Name
    bpy.ops.object.mode_set(mode='OBJECT')
    
    # Get material
    if bpy.data.materials.get(mName) is not None:
        mat = bpy.data.materials[mName]
    else:
        # create material
        mat = bpy.data.materials.new(name = mName)
    mat.name = mName
    # Assign it to object
    if len(obj.data.materials):
        # assign to 1st material slot
        obj.data.materials[0] = mat
    else:
        # no slots
        obj.data.materials.append(mat)
    
    return {'FINISHED'}


#-++++++++++++++++++++++++++++++++++++++++++
# Function for creating uv maps
#-------------------------------------------
def createUvs(ctx, numOfUvs):
    obj = ctx.active_object
    LmAngle = ctx.scene.Lm_Angle
    LmSpacing = ctx.scene.Lm_Spacing
    # Delete all existing uv maps
    for uv in obj.data.uv_layers:
        obj.data.uv_layers.active = uv
        bpy.ops.mesh.uv_texture_remove()
    
    # Decide if only to create 1 or both uv maps
    if numOfUvs == 1:
        try:
            obj.data.uv_textures[0].name = "LightMap"
        except:
            bpy.ops.mesh.uv_texture_add()
            obj.data.uv_textures[0].name = "LightMap"
        # Do smart project uvs for Lightmap  
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.uv.smart_project(angle_limit=LmAngle, island_margin=LmSpacing)
        bpy.ops.object.mode_set(mode='OBJECT')
    else:
        try:
            obj.data.uv_textures[0].name = "BaseMaterial"
        except:
            bpy.ops.mesh.uv_texture_add()
            obj.data.uv_textures[0].name = "BaseMaterial"
        try:
            obj.data.uv_textures[1].name = "LightMap"
        except:
            bpy.ops.mesh.uv_texture_add()
            obj.data.uv_textures[1].name = "LightMap"
        # do smart project uvs for Lightmap  
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.uv.smart_project(angle_limit=LmAngle, island_margin=LmSpacing)
        bpy.ops.object.mode_set(mode='OBJECT')    
    return {'FINISHED'}


#-++++++++++++++++++++++++++++++++++++++++++
# Function for scaling uv maps
#-------------------------------------------
def scaleUvs(ctx):
    obj = ctx.active_object
    ScaleX = ctx.scene.UV_ScaleX
    ScaleY = ctx.scene.UV_ScaleY
  
    bpy.context.tool_settings.mesh_select_mode = (False, False, True)
    bpy.context.tool_settings.uv_select_mode = 'ISLAND'
    
    #Scale islands
    bpy.ops.transform.resize(value=(ScaleX, ScaleY, 1),constraint_axis=(False, False, False),constraint_orientation='GLOBAL', mirror=False,proportional='DISABLED', proportional_edit_falloff='SMOOTH',proportional_size=1, snap=False, snap_target='CLOSEST',snap_point=(0, 0, 0), snap_align=False, snap_normal=(0, 0, 0),texture_space=False, release_confirm=False)
    
    return {'FINISHED'}

class MyPropertyGroup(bpy.types.PropertyGroup):
    sProp = bpy.props.BoolProperty()
    bProp = bpy.props.BoolProperty()
    iProp = bpy.props.IntProperty(
        name = "Min scale", 
        description = "Number of object scales to be exported",
        min = 1,
        max = 9,
        default= 3,
        subtype='FACTOR')
    exPath = bpy.props.StringProperty(
        name="",
        description="Export path:",
        default="",
        maxlen=1024,
        subtype='DIR_PATH')


#-++++++++++++++++++++++++++++++++++++++++++
# Class that creates Custom operator
#-------------------------------------------
class CustomOperator(bpy.types.Operator):
    bl_description = ""
    bl_idname = "tools.cust_opt"
    bl_label = "Button"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'
    opType = bpy.props.StringProperty()
    iProp =IntProperty(
        name = "Lm Angle", 
        description = "Angle from 0  to 89",
        min = 0,
        max = 89)
    
    def execute(self, context):
        obj = context.active_object
        
        if self.opType == "LIGHTMAP":
            createUvs(context, 1)
        elif self.opType == "BASEMATERIAL_LIGHTMAP":
            createUvs(context, 2)
        elif self.opType == "CREATE_MATERIAL":
            createMat(context)
        elif self.opType == "ICON":
            renderIcon(context,self)
        elif self.opType == "FBX_EXPORT":
            exportFbx(context,self)
        elif self.opType == "SCALE_UV":
            scaleUvs(context)
        elif self.opType == "FIX_VERTS":
            fixVerts(context)
        return {'FINISHED'}


#-++++++++++++++++++++++++++++++++++++++++++
# Class that creates Maniaplanet panel
#-------------------------------------------
class Maniaplanet_Panel(bpy.types.Panel):
    """Creates Maniaplanet Panel in Tools part of 3d View"""
    bl_label = "Maniaplanet Addon"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'
    bl_category = "Tools"

    # Function that draws Maniaplanet panel
    def draw(self, context):
        layout = self.layout
        
        split = layout.split()
        col = split.column(align = True)
        
        col.label(text="Uvs:", icon="GROUP_UVS")
        col.prop(context.scene,'Lm_Angle', toggle=True)
        col.prop(context.scene,'Lm_Spacing', toggle=True)
        lmBtn = col.operator(operator="tools.cust_opt", text="Lightmap")
        lmBtn.opType = "LIGHTMAP"
        uvBtn = col.operator(operator="tools.cust_opt", text="Lmap & Bmap")
        uvBtn.opType = "BASEMATERIAL_LIGHTMAP"
        
        split = layout.split()
        col = split.column(align = True)
        
        col.label(text="Material:", icon="MATERIAL_DATA")
        col.prop(context.scene, 'Mat_Name', toggle=True)
        matBtn = col.operator(operator="tools.cust_opt", text="Create mat")
        matBtn.opType = "CREATE_MATERIAL"
        
        split = layout.split()
        col = split.column(align = True)
        
        col.label(text="Icon:", icon="SCENE")
        col.prop(context.scene, 'ViewType', toggle=True)
        col.prop(context.scene, 'RenderIconSize', toggle=True)
        icoBtn = col.operator(operator="tools.cust_opt", text="Create Icon")
        icoBtn.opType = "ICON"

        col.label(text="Export:", icon="SCENE")
        col.prop(context.scene.my_prop_grp, 'sProp', text="Use scene as folder name")
        col.prop(context.scene.my_prop_grp, 'bProp', text="Enable scales")
        sub = col.row()
        sub.prop(context.scene.my_prop_grp, 'iProp',  toggle=False)
        sub.enabled = context.scene.my_prop_grp.bProp
        sub = col.row()
        col.prop(context.scene.my_prop_grp, 'exPath', text="") 

        expFbxBtn =  col.operator(operator="tools.cust_opt", text="Export")
        expFbxBtn.opType = "FBX_EXPORT"
        
        split = layout.split()
        col = split.column(align = True)
        
        col.label(text="Mesh:", icon="OBJECT_DATAMODE")
        fixVertsBtn =  col.operator(operator="tools.cust_opt", text="Fix vertecies")
        fixVertsBtn.opType = "FIX_VERTS"


#-++++++++++++++++++++++++++++++++++++++++++
# Class that creates Maniaplanet panel in UV TOOLS
#-------------------------------------------
class Maniaplanet_Panel_UV(bpy.types.Panel):
    """Creates Maniaplanet Panel in Tools part of 3d View"""
    bl_label = "Maniaplanet Addon"
    bl_space_type = 'IMAGE_EDITOR'
    bl_region_type = 'TOOLS'
    bl_category = "Tools"

    # Function that draws Maniaplanet panel inside uv tools
    def draw(self, context):
        layout = self.layout
        
        split = layout.split()
        col = split.column(align = True)
        
        col.label(text="Uvs:", icon="GROUP_UVS")
        col.prop(context.scene,'UV_ScaleX', toggle=True)
        col.prop(context.scene,'UV_ScaleY', toggle=True)
        uvScaleBtn = col.operator(operator="tools.cust_opt", text="ScaleUv")
        uvScaleBtn.opType = "SCALE_UV"
        
        split = layout.split()
        col = split.column(align = True)


#-++++++++++++++++++++++++++++++++++++++++++
# Funtion to register Maniaplanet addon
#-------------------------------------------
def register():
    bpy.utils.register_class(Maniaplanet_Panel)
    bpy.utils.register_class(CustomOperator)
    bpy.utils.register_class(Maniaplanet_Panel_UV)
    bpy.utils.register_class(MyPropertyGroup)
    bpy.types.Scene.my_prop_grp = bpy.props.PointerProperty(type=MyPropertyGroup)

    # Integer property for Number of object scales
    bpy.types.Scene.Exp_Scales = bpy.props.IntProperty(
        name = "Export Scales", 
        description = "Num of scales",
        min = 1,
        max = 7,
        default = 7)   
    # Integer property for Lightmap angle
    bpy.types.Scene.Lm_Angle = bpy.props.IntProperty(
        name = "Lm Angle", 
        description = "Angle from 0  to 89",
        min = 1,
        max = 89,
        default = 45)
    #bpy.context.scene['Lm_Angle'] = 45
    # Float property for Lightmap Island spacing 
    bpy.types.Scene.Lm_Spacing = bpy.props.FloatProperty(
        name = "Lm spacing", 
        description = "Defines how much space there will be betwean islands",
        min = 0.1,
        max = 100,
        default = 0.3)
    # Float property for Scaling uv map on X axes
    bpy.types.Scene.UV_ScaleX = bpy.props.FloatProperty(
        name = "UV scale X", 
        description = "Scales uv map on X axes",
        min = 0.01,
        max = 100.0,
        default = 0.0)
    # Float property for Scaling uv map on Y axes
    bpy.types.Scene.UV_ScaleY = bpy.props.FloatProperty(
        name = "UV scale Y", 
        description = "Scales uv map on Y axes",
        min = 0.01,
        max = 100.0,
        default = 0.0)
    # String property to store Material name
    bpy.types.Scene.Mat_Name = bpy.props.StringProperty(
        name = "Name",
        description = "Change the name of the material",
        default = "Material_name")
    # Enum property for icon view type
    bpy.types.Scene.ViewType = bpy.props.EnumProperty(
        items = [('id_0', 'Current', 'Use current 3d view'),
                ('id_1', 'Closest', 'Zoom in object  as close as possible')],
        name = "View",
        description = "View what will be used for icon rendering",
        default = "id_0")
    # Enum property for icon size
    bpy.types.Scene.RenderIconSize = bpy.props.EnumProperty(
        items = [('id_0', '16x16', '16x16px icon size'),
                ('id_1', '32x32', '32x32px icon size'),
                ('id_2', '64x64', '64x64px icon size'),
                ('id_3', '128x128', '128x128px icon size'),
                ('id_4', '256x256', '256x256px icon size')],
        name = "Size",
        description = "Icon size in pixels",
        default = "id_2")


#-++++++++++++++++++++++++++++++++++++++++++
# Function to unregister Maniaplanet addon  
#-------------------------------------------
def unregister():
    bpy.utils.unregister_class(Maniaplanet_Panel)
    bpy.utils.unregister_class(CustomOperator)
    bpy.utils.unregister_class(Maniaplanet_Panel_UV)
    bpy.utils.register_class(MyPropertyGroup)
    del bpy.types.Scene.my_prop_grp
    del bpy.types.Scene.Lm_Angle
    del bpy.types.Scene.Lm_Spacing
    del bpy.types.Scene.Mat_Name
    del bpy.types.Scene.ViewType
    del bpy.types.Scene.RenderIconSize
    del bpy.types.Scene.UV_ScaleY
    del bpy.types.Scene.UV_ScaleX
#Just for testing script
if __name__ == "__main__":
    register()