bl_info = {
    'name': 'Export image as DDS',
    'author': 'cmomoney@MTS',
    'version': (0, 3),
    'blender': (2, 6, 5),
    'category': 'Import-Export',
    'description' : 'Saves images as DDS file via Nvidia tools',
    'location': 'Image',
    'wiki_url': ''}

#Save images as DDS file using Nvidia tools.

    # ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

import bpy, struct, os
from bpy.props import *

def nvidiaGet():
    spaths = bpy.utils.script_paths()
    for rpath in spaths:
        tpath = rpath + '\\addons\\nvidia\\nvidia_dds.exe'
        if os.path.exists(tpath):
            npath = '"' + tpath + '"'
            return npath
    return None

class Save_DDS(bpy.types.Operator):
    bl_idname = 'save.dds'
    bl_label = 'Save DDS'
    bl_description = "Save image as a DDS file"
    bl_options = {'REGISTER', 'UNDO'}       
        
    filepath = StringProperty(name="File path", description="File filepath used for saveing .dds file", maxlen=1024, default="")
    filter_folder = BoolProperty(name="Filter folders", description="", default=True, options={'HIDDEN'})
    filter_glob = StringProperty(default="*.dds", options={'HIDDEN'})
    bpy.types.Scene.cuda = BoolProperty(name=" ",default=0,description="Enable CUDA for fast DDS converting..")
    dxtType = bpy.props.EnumProperty(items=[('DXT Type', 'DXT Type', 'DXT Type'),('DXT1(no alpha)', 'DXT1(no alpha)', 'DXT1(no alpha)'), ('DXT5(with alpha)', 'DXT5(with alpha)', 'DXT5(with alpha)')], default='DXT1(no alpha)')
    DDSPATH = nvidiaGet()
    
    def execute(self, context):
        if self.DDSPATH:
            self.exportfile()
        else:
            self.report({'ERROR'}, 'Nvidia tools not found, please install.')
        return{'FINISHED'}
        
    def invoke(self, context, event):
        wm = context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}
    
    def exportfile(self):            
                
        def dds_convert(img):
            if bpy.context.scene.cuda == 0:
                dxt5=" -fast -alpha -nocuda -bc3 "
                dxt1=" -fast -nocuda -bc1 "
            else:
                dxt5=" -alpha -bc3 "
                dxt1=" -bc1 "
                
            if img[-4:] in ['.png','.PNG','.tga','.TGA','.dds','.DDS']:
                input = img
                output = self.filepath+".dds"
                output2='"'+output+'"'
                input2='"'+input+'"'
                if self.dxtType == 'DXT1(no alpha)':
                    print('"'+self.DDSPATH+dxt1+" "+input2+" "+output2+'"')
                    os.system('"'+self.DDSPATH+dxt1+" "+input2+" "+output2+'"')  # Convert to DXT 1 (no alpha)
                elif self.dxtType == 'DXT5(with alpha)':
                    os.system('"'+self.DDSPATH+dxt5+" "+input2+" "+output2+'"')  # Convert to DXT 1 (no alpha)                        
          
        # get the active image
        img = bpy.context.space_data.image
        if img.source != 'FILE':
            bpy.context.scene.render.image_settings.file_format = 'TARGA'
            if self.dxtType == 'DXT5(with alpha)':
                bpy.context.scene.render.image_settings.color_mode = 'RGBA'
            else:
                bpy.context.scene.render.image_settings.color_mode = 'RGB'
            img.save_render(self.filepath+'.tga')
            dds_convert(self.filepath+'.tga')
            img.filepath = self.filepath+'.dds'
            img.source = 'FILE'
            os.remove(self.filepath+'.tga')
            img.name = os.path.split(self.filepath)[1]
        elif os.path.exists(self.filepath+".dds"):
            os.remove(self.filepath+'.dds')
            bpy.context.scene.render.image_settings.file_format = 'TARGA'
            if self.dxtType == 'DXT5(with alpha)':
                bpy.context.scene.render.image_settings.color_mode = 'RGBA'
            else:
                bpy.context.scene.render.image_settings.color_mode = 'RGB'
            img.save_render(self.filepath+'.tga')   
            dds_convert(self.filepath+'.tga')
            img.filepath = self.filepath+'.dds'
            os.remove(self.filepath+'.tga')            
            img.name = os.path.split(self.filepath)[1]
        elif os.path.exists(self.filepath+".dds") == False:
            bpy.context.scene.render.image_settings.file_format = 'TARGA'
            if self.dxtType == 'DXT5(with alpha)':
                bpy.context.scene.render.image_settings.color_mode = 'RGBA'
            else:
                bpy.context.scene.render.image_settings.color_mode = 'RGB'
            img.save_render(self.filepath+'.tga')    
            dds_convert(self.filepath+'.tga')
            img.filepath = self.filepath+'.dds'
            os.remove(self.filepath+'.tga')
            img.name = os.path.split(self.filepath)[1] 

def menu_func(self, context):
    #if bpy.context.space_data.image:    
    self.layout.operator(Save_DDS.bl_idname, text="Save as DDS")

def menu_func2(self, context):
    self.layout.operator(Save_DDS.bl_idname, text="Save as DDS")

def register():
    bpy.utils.register_class(Save_DDS)
    bpy.types.IMAGE_MT_image.append(menu_func)
    
def unregister():
    bpy.utils.unregister_class(Save_DDS)
    bpy.types.IMAGE_MT_image.remove(menu_func)
        
if __name__ == "__main__":
    register()                 